@@ -741,7 +741,7 @@ journalFilePathFromOpts opts = do
   case mbpaths of
     Just paths -> return paths
     Nothing -> do
-      f <- defaultJournalPath
+      f <- defaultExistingJournalPath
       return $ NE.fromList [f]
 
 -- | Like journalFilePathFromOpts, but does not use defaultJournalPath
@@ -755,7 +755,7 @@ journalFilePathFromOptsNoDefault opts = do
 expandPathPreservingPrefix :: FilePath -> PrefixedFilePath -> IO PrefixedFilePath
 expandPathPreservingPrefix d prefixedf = do
   let (p,f) = splitReaderPrefix prefixedf
-  f' <- expandPath d f
+  f' <- expandPathOrGlob d f
   return $ case p of
     Just p'  -> (show p') ++ ":" ++ f'
     Nothing -> f'
